"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generate = exports.generateGetters = exports.getter = exports.camelCase = exports.inject = exports.generateCSS = exports.renderClassName = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _templateObject = _taggedTemplateLiteral(["\n    .", "{\n      ", ":", "\n    }"], ["\n    .", "{\n      ", ":", "\n    }"]),
    _templateObject2 = _taggedTemplateLiteral(["\n    @media ", "{\n      ", "\n    }"], ["\n    @media ", "{\n      ", "\n    }"]);

var _ramda = require("ramda");

var _ramda2 = _interopRequireDefault(_ramda);

var _oneLineTrim = require("common-tags/lib/oneLineTrim");

var _oneLineTrim2 = _interopRequireDefault(_oneLineTrim);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _taggedTemplateLiteral(strings, raw) { return Object.freeze(Object.defineProperties(strings, { raw: { value: Object.freeze(raw) } })); } /* eslint-disable no-console */
/* global process */

/**
 * Strip "@"s and ":"s from a string
 */

var strip = _ramda2.default.replace(/[@:]/g, "");

/**
 * Strip "@"s and ":"s from a string
 */

var renderClassName = exports.renderClassName = _ramda2.default.compose(_ramda2.default.join("--"), _ramda2.default.map(strip));

/**
 * Generate CSS for single rule
 */

var rulesReducer = _ramda2.default.curry(function (declaration, makeSelector, accum, rule) {
  var key = rule[0];
  var value = rule[1] || key;

  var ruleCSS = (0, _oneLineTrim2.default)(_templateObject, makeSelector(declaration, key), declaration, value);

  return _ramda2.default.concat(accum, ruleCSS);
});

/**
 * Generate CSS for pseudo class/element
 */

var pseudoReducer = _ramda2.default.curry(function (declaration, rules, accum, pseudo) {

  var pseudoCSS = _ramda2.default.reduce(rulesReducer(declaration, function (dec, rule) {
    return dec + "--" + rule + "--" + strip(pseudo) + pseudo;
  }), "", rules);

  return _ramda2.default.concat(accum, pseudoCSS);
});

/**
 * Generates CSS for a media query
 */

var mediaQueryReducer = _ramda2.default.curry(function (declaration, rules, accum, _ref) {
  var _ref2 = _slicedToArray(_ref, 2);

  var mediaKey = _ref2[0];
  var query = _ref2[1];


  var rulesCSS = _ramda2.default.reduce(rulesReducer(declaration, function (dec, rule) {
    return dec + "--" + rule + "--" + strip(mediaKey);
  }), "", rules);

  var mediaQueryCSS = (0, _oneLineTrim2.default)(_templateObject2, query, rulesCSS);

  return _ramda2.default.concat(accum, mediaQueryCSS);
});

/**
 * Generate CSS for a signle definition
 */

var definitionReducer = function definitionReducer(accum, _ref3) {
  var _ref4 = _slicedToArray(_ref3, 2);

  var declaration = _ref4[0];
  var _ref4$ = _ref4[1];
  var rules = _ref4$.rules;
  var media = _ref4$.media;
  var pseudo = _ref4$.pseudo;


  var vanillaRulesCSS = _ramda2.default.reduce(rulesReducer(declaration, function (dec, rule) {
    return dec + "--" + rule;
  }), "", rules);

  var pseudoCSS = _ramda2.default.reduce(pseudoReducer(declaration, rules), "", pseudo || []);

  var mediaQueryRulesCSS = _ramda2.default.reduce(mediaQueryReducer(declaration, rules), "", media || []);

  return "" + accum + vanillaRulesCSS + pseudoCSS + mediaQueryRulesCSS;
};

/**
 * Generate CSS markup
 * @param {object} config
 * @returns {string}
 */

var generateCSS = exports.generateCSS = function generateCSS(config) {
  var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var css = _ramda2.default.reduce(definitionReducer, "", config);

  return "" + (options.globals || "") + css;
};

/**
 * Inject CSS into <head>
 * @param {string}
 * @returns {void}
 */

var inject = exports.inject = function inject(css) {
  var style = document.createElement("style");
  style.type = "text/css";
  style.appendChild(document.createTextNode(css));

  document.head.appendChild(style);
};

/**
 * Convert kebab-case to camelCase
 */

var camelCase = exports.camelCase = function camelCase(string) {
  return _ramda2.default.replace(/-([a-z])/g, function (_, match) {
    return _ramda2.default.toUpper(match);
  }, string);
};

/**
 * Retrieve classname from config
 */

var getter = exports.getter = function getter(_ref5) {
  var _ref6 = _slicedToArray(_ref5, 2);

  var def = _ref6[0];
  var _ref6$ = _ref6[1];
  var rules = _ref6$.rules;
  var media = _ref6$.media;
  var pseudo = _ref6$.pseudo;
  return function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var key = args[0];
    var arg = args[1];

    // This is a convenience for being able to call the getter function with
    // arguments but without generating no output.
    // This is useful when passing props directly

    if (key === null) return null;

    if (process.env.NODE_ENV !== "production") {
      if (!_ramda2.default.find(_ramda2.default.propEq(0, key), rules)) {
        return console.error("Couldn't find '" + key + "' key for '" + def + "' definition.");
      }

      if (arg) {
        var isPseudo = arg.startsWith(":");
        var isMedia = arg.startsWith("@");

        if (isMedia && !_ramda2.default.find(_ramda2.default.propEq(0, arg), media)) {
          return console.error("Couldn't find '" + media + "' media query for '" + def + "' definition.");
        }

        if (isPseudo && !_ramda2.default.find(_ramda2.default.equals(arg), pseudo)) {
          return console.error("Couldn't find '" + pseudo + "' psedo class/element for '" + def + "' definition.");
        }
      }
    }

    return renderClassName([def].concat(args));
  };
};

/**
 * Create getter functions
 */

var generateGetters = exports.generateGetters = function generateGetters(defs) {
  return _ramda2.default.reduce(function (acc, def) {
    return _ramda2.default.assoc(camelCase(def[0]), getter(def), acc);
  }, {}, defs);
};

/**
 * Generate CSS and getter functions
 */

var generate = exports.generate = function generate(config, options) {
  return {
    css: generateCSS(config, options),
    getters: generateGetters(config)
  };
};

/**
 * Main
 */

exports.default = function (config, options) {
  var _generate = generate(config, options);

  var css = _generate.css;
  var getters = _generate.getters;

  inject(css);
  return getters;
};